--[[
This class controls the arrangement of the ActionButtonNone and ActionButton
instances for a UnitGroupHeader.

There are one or more IActionSetLayout instances allocated for each
UnitGroupHeader.  How many depends on whether the layout is changed while
in-game.  The only layouts allocated are ones that were selected in-game.  Each
UnitGroupHeader keeps track of the active ActionSetLayout to use.
--]]
uab.classes.OvalLayout = uab.mixins.aceOO.Class(IActionSetLayout)

--[[
This table defines the placement order for the buttons assuming
a specific starting point.  Each row is positioned, then the next
row.  Iterate over the button order positions until all have been
used, then go to the next row.  This means position 6 buttons in
the first row, 12 in the second, and 6 in the third.

To use this table when not starting at the specific starting point
use the buttonOrderSequence table to find the first index for each
row for a specific starting point.
--]]
uab.classes.OvalLayout.constants = {}
uab.classes.OvalLayout.constants.rowButtonStart = { 1, 7, 19 }
uab.classes.OvalLayout.constants.rowButtons = { 6, 12, 6 }
uab.classes.OvalLayout.constants.buttonOrder = {
    [uab.constants.anchor.left] = { row1 = { "l2", "u2l1", "u2r1", "r2", "d2r1", "d2l1" },
          row2 = { "l4", "u2l3", "u4l2", "u4", "u4r2", "u2r3", "r4", "d2r3", "d4r2", "d4", "d4l2", "d2l3" },
          row3 = { "l6", "u2l5", "u2r5", "r6", "d2r5", "d2l5" }
    },
    [uab.constants.anchor.top] = { row1 = { "u2", "u1r2", "d1r2", "d2", "d1l2", "u1l2" },
          row2 = { "u4", "u3r2", "u2r4", "r4", "d2r4", "d3r2", "d4", "d3l2", "d2l4", "l4", "u2l4", "u3l2" },
          row3 = { "u6", "u5r2", "d5r2", "d6", "d5l2", "u5l2" }
    }
}

uab.classes.OvalLayout.constants.buttonOrderSequence = {
    [uab.constants.anchor.left] = {
          [uab.constants.placement.left] = { 1, 1, 1 },
          [uab.constants.placement.topleft] = { 2, 2, 2 },
          [uab.constants.placement.topright] = { 3, 4, 3 },
          [uab.constants.placement.right] = { 4, 7, 4 },
          [uab.constants.placement.bottomright] = { 5, 8, 5 },
          [uab.constants.placement.bottomleft] = { 6, 10, 6 }
    },
    [uab.constants.anchor.top] = {
          [uab.constants.placement.top] = { 1, 1, 1 },
          [uab.constants.placement.topright] = { 2, 2, 2 },
          [uab.constants.placement.bottomright] = { 3, 4, 3 },
          [uab.constants.placement.bottom] = { 4, 7, 4 },
          [uab.constants.placement.bottomleft] = { 5, 8, 5 },
          [uab.constants.placement.topleft] = { 6, 10, 6 }
    }
}

function uab.classes.OvalLayout.prototype:init(settings)
	uab.classes.OvalLayout.super.prototype.init(self)
	
    self.config = uab.classes.OvalLayoutConfig:new(self, settings)
end

function uab.classes.OvalLayout.prototype:ArrangeButtons(cancelbtn, actionbtns)
	--[[
	First, reset all positioning information
	--]]
    cancelbtn:Frame():ClearAllPoints()
    self:FormatCancelBtn(cancelbtn)

	for idx = 1, 24, 1 do
	    if actionbtns[idx] then
	        actionbtns[idx]:Frame():ClearAllPoints()
            self:FormatActionBtn(actionbtns[idx])
		end
	end

	--[[
    Now, position the cancel button centered relative to the header (the
    parent of the cancel button)
    --]]
    cancelbtn:Frame():SetPoint("CENTER", 0, 0)

    self:PositionRow(1, cancelbtn:Frame(), actionbtns)
    self:PositionRow(2, cancelbtn:Frame(), actionbtns)
    self:PositionRow(3, cancelbtn:Frame(), actionbtns)
end

function uab.classes.OvalLayout.prototype:PositionRow(row, cancelframe, actionbtns)
    local order = uab.classes.OvalLayout.constants.buttonOrder[self.config.settings.anchor]["row"..row]
    local sequence = uab.classes.OvalLayout.constants.buttonOrderSequence[self.config.settings.anchor]
    local startpos = self.config.settings.startLocation
    local buttonIdx = uab.classes.OvalLayout.constants.rowButtonStart[row]

    if self.config.settings.growClockwise == "true" then
        for idx = sequence[startpos][row], uab.classes.OvalLayout.constants.rowButtons[row], 1 do
            self:PositionButton(order[idx], buttonIdx, actionbtns[buttonIdx], cancelframe)
            buttonIdx = buttonIdx + 1
        end
        
        local endpos = sequence[startpos][row] - 1
        
        for idx = 1, endpos, 1 do
            self:PositionButton(order[idx], buttonIdx, actionbtns[buttonIdx], cancelframe)
            buttonIdx = buttonIdx + 1
        end
    else
        for idx = sequence[startpos][row], 1, -1 do
            self:PositionButton(order[idx], buttonIdx, actionbtns[buttonIdx], cancelframe)
            buttonIdx = buttonIdx + 1
        end
        
        local endpos = sequence[startpos][row] + 1
        
        for idx = uab.classes.OvalLayout.constants.rowButtons[row], endpos, -1 do
            self:PositionButton(order[idx], buttonIdx, actionbtns[buttonIdx], cancelframe)
            buttonIdx = buttonIdx + 1
        end
    end
end

function uab.classes.OvalLayout.prototype:PositionButton(orderCode, btnidx, btn, center)
    if not btn then return end
    
    local vMod = 0
    local hMod = 0

    tmpCode = orderCode        
    if string.sub(orderCode, 1, 1) == "u" then
        vMod = tonumber(string.sub(orderCode, 2, 2)) * 0.5
        tmpCode = string.sub(orderCode, 3)
    end

    if string.sub(tmpCode, 1, 1) == "d" then
        vMod = tonumber(string.sub(tmpCode, 2, 2)) * -0.5
        tmpCode = string.sub(tmpCode, 3)
    end

    if string.sub(tmpCode, 1, 1) == "r" then
        hMod = tonumber(string.sub(tmpCode, 2, 2)) * 0.5
        tmpCode = string.sub(tmpCode, 3)
    end

    if string.sub(tmpCode, 1, 1) == "l" then
        hMod = tonumber(string.sub(tmpCode, 2, 2)) * -0.5
        tmpCode = string.sub(tmpCode, 3)
    end

    local x = (uab.constants.buttonWidth + 1) * hMod
    local y = (uab.constants.buttonHeight + 1) * vMod
    
    btn:Frame():SetPoint("CENTER", center, "CENTER", x, y)
end

function uab.classes.OvalLayout.prototype:GetConfig()
	return self.config
end

function uab.classes.OvalLayout.prototype:SetSettings(settings)
    self.config:SettingsChanged(settings)
end

function uab.classes.OvalLayout.prototype:FormatCancelBtn(cancelbtn)
    -- Nothing to do here at this time.  Would be a good idea to resize
    -- the cancel button frame, hide the text frame for the button frame
end

function uab.classes.OvalLayout.prototype:FormatActionBtn(btn)
    -- Nothing to do here at this time.  Would be a good idea to resize
    -- the cancel button frame, hide the text frame for the button frame
end

