--[[
This controls the configuration settings (and display of a sample set of
actions based on those settings) for each ActionSetLayout.  Each
BarActionSetLayout is associated with an BarActionSetLayoutConfig, and each
BarActionSetLayoutConfig refers to that same BarActionSetLayout instance.
--]]
uab.classes.SquareLayoutConfig = uab.mixins.aceOO.Class(IActionSetLayoutConfig, uab.mixins.aceEvent)
uab.classes.SquareLayoutConfig.count = 0

function uab.classes.SquareLayoutConfig.StartLocationDisplayValue(value)
    if value == uab.constants.placement.top then
        return "Top"
    elseif value == uab.constants.placement.topright then
        return "Top Right"
    elseif value == uab.constants.placement.topleft then
        return "Top Left"
    elseif value == uab.constants.placement.bottom then
        return "Bottom"
    elseif value == uab.constants.placement.bottomright then
        return "Bottom Right"
    elseif value == uab.constants.placement.bottomleft then
        return "Bottom Left"
    elseif value == uab.constants.placement.left then
        return "Left"
    else
        return "Right"
    end
end

function uab.classes.SquareLayoutConfig.prototype:init(layout, settings)
    uab.classes.SquareLayoutConfig.super.prototype.init(self)

	self.frames = {}

	self.settings = settings

	self:SettingsChanged()

	uab.classes.SquareLayoutConfig.count = uab.classes.SquareLayoutConfig.count + 1
	self.number = uab.classes.SquareLayoutConfig.count
end

function uab.classes.SquareLayoutConfig.prototype:GetFrame(parent)
	if not self.frame then
	    self:CreateFrame(parent)
	end
	
	return self.frame
end

function uab.classes.SquareLayoutConfig.prototype:SettingsChanged(settings)
	if settings then
		self.settings = settings
	end

    self.settings.growClockwise = (settings and settings.growClockwise) or "true"
    self.settings.startLocation = (settings and settings.startLocation) or uab.constants.placement.top

	if self.frame then
		--[[
		Update the controls in the frame to show the new settings
		--]]
    	self:SetGrowClockwise(self.settings.growClockwise)
    	self:SetStartLocation(self.settings.startLocation)
	end
end

function uab.classes.SquareLayoutConfig.prototype:UseDefaults(bypassNotification)
	if not self.settings then
	    self.settings = {}
	end

    self.settings.growClockwise = "true"
    self.settings.startLocation = uab.constants.placement.top
	
    self:SettingsChanged()
	
    if not bypassNotification then
        uab.main:LayoutSettingsChanged()
    end
end

function uab.classes.SquareLayoutConfig.prototype:MenuOptionArgs()
	--[[
	Build an AceOptions table of the options to be used to configure this
	config instance.  This table is inserted into a parent menu as a set of
	options.  If there are several options, just define them at the top level
	of the returned table.  Do not create a 'group' at the top level to enclose
	the options.
	--]]
	
	local options = {
        clockwise = {
            type = 'toggle',
            name = "Add buttons clockwise",
            desc = "As buttons are added, then should be added around the square in clockwise order",
            get = function()
                return self.settings.growClockwise == "true"
            end,
            set = function(v)
                if v then
					self:SetGrowClockwise("true")
				else
				    self:SetGrowClockwise("false")
                end
            end,
        },
        location = {
            type = 'text',
            name = "First button placement",
            desc = "Placement of the first button, relative to the cancel button",
            get = function()
                return uab.classes.SquareLayoutConfig.StartLocationDisplayValue(self.settings.startLocation)
            end,
            set = function(placement)
                if placement == uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.topleft) then
                    self:SetStartLocation(uab.constants.placement.topleft)
				elseif placement == uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.top) then
                	self:SetStartLocation(uab.constants.placement.top)
				elseif placement == uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.topright) then
                	self:SetStartLocation(uab.constants.placement.topright)
				elseif placement == uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottomleft) then
                	self:SetStartLocation(uab.constants.placement.bottomleft)
				elseif placement == uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottom) then
                	self:SetStartLocation(uab.constants.placement.bottom)
				elseif placement == uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottomright) then
                	self:SetStartLocation(uab.constants.placement.bottomright)
				elseif placement == uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.left) then
                	self:SetStartLocation(uab.constants.placement.left)
				elseif placement == uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.right) then
                	self:SetStartLocation(uab.constants.placement.right)
				end
            end,

            validate = {
				uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.topleft),
				uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.top),
				uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.topright),
				uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottomleft),
				uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottom),
				uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.bottomright),
				uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.left),
				uab.classes.SquareLayoutConfig.StartLocationDisplayValue(uab.constants.placement.right),
			},
        },
	}
	
	return options
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.SquareLayoutConfig.prototype:CreateFrame(parent)
	self.frame = CreateFrame("Frame", "uabSqaureOptions"..self.number, parent or UIParent, "UAB_SquareLayoutOptionsTemplate")

	self:SetupFrame()
end

function uab.classes.SquareLayoutConfig.prototype:SetupFrame()
	self.frames = {}

    self.frames.growDirection = getglobal(self.frame:GetName().."GrowDirection")
    self.frames.growDirection.config = self
	
    getglobal(self.frames.growDirection:GetName().."Text"):SetText("Add buttons clockwise")
	self.frames.growDirection:SetScript("OnClick", function(self, button, down) self.config:GrowDirectionButtonClicked(self) end)

    self.frames.startLocation = getglobal(self.frame:GetName().."StartLocation")
    self.frames.startLocation.config = self
	
    uab.classes.SquareLayoutConfig.InsideInit = self
    
    UIDropDownMenu_Initialize(self.frames.startLocation,
        function()
            if uab.classes.SquareLayoutConfig.InsideInit then
                uab.classes.SquareLayoutConfig.InsideInit:StartLocationListInitialize()
            else
                this:GetParent().config:StartLocationListInitialize()
            end
        end)
	UIDropDownMenu_SetWidth(110, self.frames.startLocation)
	UIDropDownMenu_SetButtonWidth(24, self.frames.startLocation)
	UIDropDownMenu_JustifyText("LEFT", self.frames.startLocation)

    uab.classes.SquareLayoutConfig.InsideInit = nil
    
	self:SetGrowClockwise(self.settings.growClockwise)
	self:SetStartLocation(self.settings.startLocation)
end

function uab.classes.SquareLayoutConfig.prototype:StartLocationListInitialize()
    local index = 1
    local selected = nil

    local options = {}

    table.insert(options, {	text = "Top Left",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.topleft }})
    table.insert(options, {	text = "Top",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.top }})
    table.insert(options, {	text = "Top Right",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.topright }})
    table.insert(options, {	text = "Right",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.right }})
    table.insert(options, {	text = "Bottom Right",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.bottomright }})
    table.insert(options, {	text = "Bottom",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.bottom }})
    table.insert(options, {	text = "Bottom Left",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.bottomleft }})
    table.insert(options, {	text = "Left",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectStartLocation", arg1 = uab.constants.placement.left }})

    for index, data in pairs(options) do
        UIDropDownMenu_AddButton(data)
        if self.settings.startLocation == data.arg2.arg1 then
            selected = index
        end
    end

    UIDropDownMenu_SetSelectedID(self.frames.startLocation, selected)
end

function uab.classes.SquareLayoutConfig.prototype:SelectStartLocation(selector, selection)
    UIDropDownMenu_SetSelectedID(self.frames.startLocation, selector:GetID())
    self:SetStartLocation(selection)
end

function uab.classes.SquareLayoutConfig.prototype:SetStartLocation(value)
    local notify = (value ~= self.settings.startLocation)
        
    self.settings.startLocation = value
    if self.frames.startLocation then
        UIDropDownMenu_SetText(uab.classes.SquareLayoutConfig.StartLocationDisplayValue(value), self.frames.startLocation)
    end
    
    if notify then
        uab.main:LayoutSettingsChanged()
    end        
end

function uab.classes.SquareLayoutConfig.prototype:GrowDirectionButtonClicked(frame)
    self:SetGrowClockwise(frame:GetChecked())
end

function uab.classes.SquareLayoutConfig.prototype:SetGrowClockwise(value)
    local newval
    
    if value and (tonumber(value) == 1 or value == "true") then
        newval = "true"
    else
        newval = "false"
    end
    
    local notify = (newval ~= self.settings.growClockwise)
        
    self.settings.growClockwise = newval
    if self.frames.growDirection then
        self.frames.growDirection:SetChecked(self.settings.growClockwise == "true")
    end

    if notify then
        uab.main:LayoutSettingsChanged()
    end        
end

